/* 
modified for INOVA 08/11/2017 Peter

Pulse sequence of 1D interferogram pure shift experiments using BIRD
Peter Kiraly

----------------------
Developed By NMR Group
School of Chemistry
University of Manchester
United Kingdom
Aug 2017
----------------------
*/

#include <standard.h>

static int	ph1[8] = {0,0,0,0, 0,0,0,0},
		ph2[8] = {1,1,2,2, 3,3,0,0},
		ph9[2] = {0,1},			
		ph5[8] = {2,0,0,2, 2,0,0,2};	

pulsesequence()
{
double 	droppts = getval("droppts"),		
	hsgt = getval("hsgt"),
	hsglvl = getval("hsglvl"),
	gstab = getval("gstab"),
	gt1 = getval("gt1"),  
	gzlvl1 = getval("gzlvl1"),
	gzlvl2 = getval("gzlvl2"),
	gzlvl3 = getval("gzlvl3"),
	pwxlvl = getval("pwxlvl"),
 	pwx = getval("pwx"),
	pw_HBIP = 2.0*getval("pw"),	
	pw_XBIP = getval("pw_XBIP"),
	pwr_HBIP = getval("tpwr"),	
	pwr_XBIP = getval("pwr_XBIP"),
	j1xh = getval("j1xh"),
	tau = (double)(1.0e-6*(floor)(1.0e6/(4.0*j1xh)));
int 	kpph = (int)(getval("kpph")); 	
char	sspul[MAXSTR], PFGflg[MAXSTR], 
	shp_XBIP[MAXSTR], shp_HBIP[MAXSTR], BIRDmode[MAXSTR];
    
	getstr("sspul",sspul); 
	getstr("PFGflg",PFGflg);
	getstr("shp_HBIP",shp_HBIP);
	getstr("shp_XBIP",shp_XBIP);
	getstr("BIRDmode",BIRDmode);

	if (fabs((sw/sw1)-(double)((int)((sw/sw1)+0.5)))> 0.01) 
	{
	text_message("WARNING: sw1 should be an integer submultiple of sw\n");
	}
	if ( at > 0.41 ) 
	{
	text_error("Error: at is too long. Don't fry the probe.. ");
	psg_abort(1);
	}
	if ( d1 < 1.0 ) 
	{
	text_error("Error: d1 is too short; duty cycle for decoupling is dangerously high.. ");
	psg_abort(1);
	}

	if (kpph==0)
	{
	settable(t1,8,ph1);
	settable(t2,8,ph2);
	settable(t5,8,ph5);
	}
	if (kpph>0)
	{
	settable(t1,kpph,ph1);
	settable(t2,kpph,ph2);
	settable(t5,kpph,ph5);
	}
	if (kpph<0)
	{
	settable(t1,-kpph,ph1);
	settable(t2,-kpph,ph2);
	settable(t5,-kpph,ph5);
	}
  	getelem(t1, ct, v1);		
  	getelem(t2, ct, v2);
	getelem(t5, ct, oph);
	mod2(ct,v9);	

status(A);

      	obspower(tpwr);
	if (BIRDmode[0]=='b') { decpower(pwr_XBIP); }
	else	{ decpower(pwxlvl); }
	txphase(zero);
	decphase(zero);
	obsoffset(tof);
	decoffset(dof);

	if (sspul[A] == 'y')
	{

	if (PFGflg[A] == 'y')
      {
	obspower(tpwr);
	delay(5.0e-5);
	zgradpulse(hsglvl,hsgt); 
        rgpulse(pw,zero,rof1,rof1);
	zgradpulse(hsglvl,hsgt); 	
      }
       	else
       	{
        obspower(tpwr-12);
	delay(5.0e-5);
        rgpulse(500*pw,zero,rof1,rof1);
        rgpulse(500*pw,one,rof1,rof1);
	}
	}
	delay(d1);
status(B);  
	rgpulse(pw,zero,rof1,rof1);
	delay(tau -gt1-gstab-2.0*rof1);
	zgradpulse(gzlvl3,gt1);
	delay(gstab);
		ifzero(v9);
			if (BIRDmode[0]=='h')
			{
			simpulse(2.0*pw,2.0*pwx,v2,one,rof1,rof1); 	 
			}
			if (BIRDmode[0]=='b')
			{
			simshaped_pulse(shp_HBIP,shp_XBIP,2.0*pw,pw_XBIP,v2,one,rof1,rof1); 	 
			}		
		elsenz(v9);
			if (BIRDmode[0]=='h')
			{
				if (2.0*pw > 2.0*pwx) { rgpulse(2.0*pw,v2,rof1,rof1); }
				else 
				{
				delay((2.0*pwx-2.0*pw)*0.5);
				rgpulse(2.0*pw,v2,rof1,rof1);
				delay((2.0*pwx-2.0*pw)*0.5);
				}
			}
			if (BIRDmode[0]=='b')
			{
				if (2.0*pw > pw_XBIP) { rgpulse(2.0*pw,v2,rof1,rof1); }
				else 
				{
				delay((pw_XBIP-2.0*pw)*0.5);
				rgpulse(2.0*pw,v2,rof1,rof1);
				delay((pw_XBIP-2.0*pw)*0.5);
				}
			}		
		endif(v9);				
	delay(d2/2.0);
	delay(0.25/sw1 -gt1-gstab-2.0*rof1);
	zgradpulse(gzlvl1+gzlvl3,gt1);
	delay(gstab);
	rgpulse(2.0*pw,one,rof1,rof1); 
	delay(0.25/sw1- gt1- gstab -2.0*rof1);	
	zgradpulse(gzlvl1+gzlvl2,gt1);	
	delay(gstab);
	rgpulse(pw,one,rof1,rof1);
	delay(tau-2.0*rof1);
	if (BIRDmode[0]=='h')
	{
	simpulse(2.0*pw,2.0*pwx,two,two,rof1,rof1); 	 
	}
	if (BIRDmode[0]=='b')
	{
	simshaped_pulse(shp_HBIP,shp_XBIP,2.0*pw,pw_XBIP,two,two,rof1,rof1); 	 
	}		
	decpower(dpwr);	
	delay(tau-2.0*rof1);
	rgpulse(pw,three,rof1,rof2);
	zgradpulse(gzlvl2,gt1);
	delay(gstab-rof2);
	delay(tau  -gt1-gstab - droppts/sw );
	delay(d2/2.0);
status(C);

}



