/* 
modified for INOVA 08/11/2017 Peter

Pulse sequence of 1D interferogram pure shift experiments using band-selective/Zangger-Sterk/PSYCHE
Peter Kiraly

----------------------
Developed By NMR Group
School of Chemistry
University of Manchester
United Kingdom
Aug 2017
----------------------
*/

#include <standard.h>

//can be used with nt=1; nt=2;(4);8;16;32;64 are advised for minor improvements
static int	ph1[64] = {0,2, 0,2, 0,2,0,2, 1,3,1,3,1,3,1,3, 0,2,0,2,0,2,0,2,1,3,1,3,1,3,1,3, 0,2,0,2,0,2,0,2,1,3,1,3,1,3,1,3, 0,2,0,2,0,2,0,2,1,3,1,3,1,3,1,3},
		ph2[64] = {0,0, 0,0, 1,1,1,1, 0,0,0,0,1,1,1,1, 0,0,0,0,1,1,1,1,0,0,0,0,1,1,1,1, 2,2,2,2,3,3,3,3,2,2,2,2,3,3,3,3, 2,2,2,2,3,3,3,3,2,2,2,2,3,3,3,3},
		ph3[64] = {0,0, 1,1, 0,0,1,1, 0,0,1,1,0,0,1,1, 2,2,3,3,2,2,3,3,2,2,3,3,2,2,3,3, 0,0,1,1,0,0,1,1,0,0,1,1,0,0,1,1, 2,2,3,3,2,2,3,3,2,2,3,3,2,2,3,3},
		ph5[64] = {0,2, 2,0, 2,0,0,2, 1,3,3,1,3,1,1,3, 0,2,2,0,2,0,0,2,1,3,3,1,3,1,1,3, 0,2,2,0,2,0,0,2,1,3,3,1,3,1,1,3, 0,2,2,0,2,0,0,2,1,3,3,1,3,1,1,3};
	
pulsesequence()
{	
double	rof3 = getval("rof3"),		
	tau_a = getval("tau_a"),
	tpwr = getval("tpwr"),
	tpwrf = getval("tpwrf"),
	pw = getval("pw"),
	pp = 2.0*pw, //getval("pp"),
	pplvl = tpwr, //getval("pplvl"),
	pplvlf = getval("pplvlf"),
	pw180_a = getval("pw180_a"),
	pwr180_a = getval("pwr180_a"),
	hsgt = getval("hsgt"),
	hsglvl = getval("hsglvl"),
	gstab = getval("gstab"),
	gt1 = getval("gt1"),  
	gzlvl1 = getval("gzlvl1"),
	gt2 = getval("gt2"),  
	gzlvl2 = getval("gzlvl2"),
	gzlvl7 = getval("gzlvl7"),	
	kp_pfgtc = getval("kp_pfgtc"),  
	sw1 = getval("sw1"),		
	droppts = getval("droppts");
int 	kpph = getval("kpph");		
char	sspul[MAXSTR],
	shp_a[MAXSTR];
  
	getstr("sspul",sspul);
	getstr("shp_a",shp_a);  

if (gzlvl7==0.0) kp_pfgtc=0.0;	


if ((kpph<0) || (kpph>64))
{
abort_message("Number of steps for phase cycling is incorrect..change kpph..");
}
if ((fmod(droppts,2))!=0.0)
{
abort_message("droppts must be an integer multiple of 2..");
}
if ( (0.25/sw1-gt1-gstab) < 0.0)
{
abort_message("chunktime is too short to accomodate gt1/gstab..");
}
if ((tau_a-rof1-gt2-gstab-2.0*kp_pfgtc)<0.0)
{
abort_message("tau_a is too short to accomodate gt2/gstab..");
}
if ( (fmod(sw/sw1,1)) !=0.0)
{
abort_message("sw1 must be an integer submultiple of sw..");
}

fprintf(stderr,"delay is %g\n",POWER_DELAY);

if (kpph==0)
{
	settable(t1,64,ph1);
	settable(t2,64,ph2);
	settable(t3,64,ph3);
	settable(t5,64,ph5);
}
else
{
	settable(t1,kpph,ph1);
	settable(t2,kpph,ph2);
	settable(t3,kpph,ph3);
	settable(t5,kpph,ph5);
}
	sub(ct, ssctr, v20);
	getelem(t1, v20, v1);
	getelem(t2, v20, v2);
	getelem(t3, v20, v3);
	getelem(t5, v20, oph);

status(A);
	txphase(zero);
        obsoffset(tof);
	obspower(tpwr);
	obspwrf(tpwrf);
	delay(0.001);
	if (sspul[0]=='y') 
	{
	zgradpulse(hsglvl,hsgt); 
	delay(gstab);
	rgpulse(pw,zero,rof1,rof1);
	zgradpulse(hsglvl,hsgt); 
	delay(0.05);
	}
	delay(d1);
status(B);
	rgpulse(pw,v1,rof1,0.0);
delay(d2/2.0);
	obspower(pplvl);	
	if (tpwrf!=pplvlf) obspwrf(pplvlf);
	delay(0.25/sw1-rof1-gt1-gstab -POWER_DELAY);
	zgradpulse(gzlvl1,gt1);
	delay(gstab-2.0*GRADIENT_DELAY);
	rgpulse(pp,v2,rof1,rof1);
	obspower(pwr180_a);
	if (tpwrf!=pplvlf) obspwrf(4095.0);
	if ( (0.25/sw1-rof1-gt1-gstab  -POWER_DELAY) > gstab )
	{
	delay(gstab-2.0*GRADIENT_DELAY);
	zgradpulse(gzlvl1,gt1);
	delay(0.25/sw1-rof1-gt1-gstab  -POWER_DELAY);
	}
	else
	{
	delay(0.25/sw1-rof1-gt1-gstab  -POWER_DELAY);
	zgradpulse(gzlvl1,gt1);
	delay(gstab-2.0*GRADIENT_DELAY);
	}
	delay(droppts/sw);
	delay(tau_a-rof1-gt2-gstab-2.0*kp_pfgtc +rof2);
	zgradpulse(gzlvl2,gt2);
	delay(gstab-2.0*GRADIENT_DELAY);
	if (gzlvl7>0.0)
	{
	rgradient('z',gzlvl7);
	delay(kp_pfgtc);
	}
	shaped_pulse(shp_a,pw180_a,v3,rof1,rof1);
	if (gzlvl7>0.0)
	{
	rgradient('z',0.0);
	delay(kp_pfgtc);
	}
	if ((tau_a-rof1-gt2-gstab) > gstab)
	{
	delay(gstab-2.0*GRADIENT_DELAY);
	zgradpulse(gzlvl2,gt2);
	delay(tau_a-rof1-gt2-gstab);
	}
	else
	{
	delay(tau_a-rof1-gt2-gstab);
	zgradpulse(gzlvl2,gt2);
	delay(gstab-2.0*GRADIENT_DELAY);
	}
delay(d2/2.0);

status(C);
}



