/* 
modified for INOVA 08/11/2017 Peter

Pulse sequence of 1D real-time pure shift experiments using band-selective/Zangger-Sterk
Peter Kiraly

----------------------
Developed By NMR Group
School of Chemistry
University of Manchester
United Kingdom
Aug 2017
----------------------
*/


#include <standard.h>


static int	ph1[64] = {0,2, 0,2, 0,2,0,2, 1,3,1,3,1,3,1,3, 0,2,0,2,0,2,0,2,1,3,1,3,1,3,1,3, 0,2,0,2,0,2,0,2,1,3,1,3,1,3,1,3, 0,2,0,2,0,2,0,2,1,3,1,3,1,3,1,3},
		ph2[64] = {0,0, 0,0, 1,1,1,1, 0,0,0,0,1,1,1,1, 0,0,0,0,1,1,1,1,0,0,0,0,1,1,1,1, 2,2,2,2,3,3,3,3,2,2,2,2,3,3,3,3, 2,2,2,2,3,3,3,3,2,2,2,2,3,3,3,3},
		ph3[64] = {0,0, 1,1, 0,0,1,1, 0,0,1,1,0,0,1,1, 2,2,3,3,2,2,3,3,2,2,3,3,2,2,3,3, 0,0,1,1,0,0,1,1,0,0,1,1,0,0,1,1, 2,2,3,3,2,2,3,3,2,2,3,3,2,2,3,3},
		ph4[64] = {0,0, 1,1, 0,0,1,1, 0,0,1,1,0,0,1,1, 2,2,3,3,2,2,3,3,2,2,3,3,2,2,3,3, 0,0,1,1,0,0,1,1,0,0,1,1,0,0,1,1, 2,2,3,3,2,2,3,3,2,2,3,3,2,2,3,3},
		ph5[64] = {0,2, 2,0, 2,0,0,2, 1,3,3,1,3,1,1,3, 0,2,2,0,2,0,0,2,1,3,3,1,3,1,1,3, 0,2,2,0,2,0,0,2,1,3,3,1,3,1,1,3, 0,2,2,0,2,0,0,2,1,3,3,1,3,1,1,3};

pulsesequence()
{	
double	rof3 = getval("rof3"),
	tau_a = getval("tau_a"),
	tau_p = getval("tau_p"),
	tau_r = 0.0,
	tpwr = getval("tpwr"),
	tpwrf = getval("tpwrf"),
	pw = getval("pw"),
	pp = 2.0*pw, //getval("pp"),
	pplvl = tpwr, //getval("pplvl"),
	pplvlf = getval("pplvlf"),
	pw180_a = getval("pw180_a"),
	pwr180_a = getval("pwr180_a"),
	hsgt = getval("hsgt"),
	hsglvl = getval("hsglvl"),
	gstab = getval("gstab"),
	gt1 = getval("gt1"),  
	gzlvl1 = getval("gzlvl1"),
	gt2 = getval("gt2"),  
	gzlvl2 = getval("gzlvl2"),
	gzlvl7 = getval("gzlvl7"),
	kp_pfgtc = getval("kp_pfgtc"),  
	ACQ_gt1 = getval("ACQ_gt1"),  
	ACQ_gzlvl1 = getval("ACQ_gzlvl1"),
	ACQ_gzlvl2 = getval("ACQ_gzlvl2"),
	//ACQ_gzlvl3 = getval("ACQ_gzlvl3"),
	//ACQ_gzlvl4 = getval("ACQ_gzlvl4"),
	ACQ_gstab = getval("ACQ_gstab"),
	kp_npoints = getval("kp_npoints"),
	droppts1 = getval("droppts1"),	
	droppts2 = getval("droppts2"),	
	kp_cycles =(double) (floor)( getval("kp_cycles") -1.0 );
	initval(kp_cycles,v10);	
int 	kpph = getval("kpph"),	
	kp_scyc_len;

char	sspul[MAXSTR],
	shp_a[MAXSTR];
  
	getstr("sspul",sspul);
	getstr("shp_a",shp_a);	

if (gzlvl7==0.0) kp_pfgtc=0.0;	
	tau_r = (tau_p - 4.0*ACQ_gt1-4.0*ACQ_gstab-pw180_a -kp_pfgtc-6.0*rof1-pp -8.0*GRADIENT_DELAY )/2.0; 

if ((kpph<0) || (kpph>64))
{
abort_message("Number of steps for phase cycling is incorrect..change kpph..");
}
if ( (0.25*kp_npoints/sw-gt1-gstab) < 0.0)
{
abort_message("chunktime is too short to accomodate gt1/gstab..");
}
if ((tau_a-rof1-gt2-gstab-2.0*kp_pfgtc)<0.0)
{
abort_message("tau_a is too short to accomodate gt2/gstab..");
}
if ( fmod( np/(2.0*(droppts1+droppts2+kp_npoints)),1.0) != 0.0)
{
abort_message("np must be an integer multiple of (droppts1+kp_npoints+droppts2)..");
}
if (kpph==0)
{
	settable(t1,64,ph1);
	settable(t2,64,ph2);
	settable(t3,64,ph3);
	settable(t4,64,ph4);
	settable(t5,64,ph5);
}
else
{
	settable(t1,kpph,ph1);
	settable(t2,kpph,ph2);
	settable(t3,kpph,ph3);
	settable(t4,kpph,ph4);
	settable(t5,kpph,ph5);
}
	getelem(t1, ct, v1);
	getelem(t2, ct, v2);
	getelem(t3, ct, v3);
	getelem(t4, ct, v4);
	getelem(t5, ct, oph);



	txphase(zero);
	xmtrphase(zero);
        obsoffset(tof);
	obspower(tpwr);
	obspwrf(tpwrf);
	delay(0.001);

	if (sspul[0]=='y') 
	{
	zgradpulse(hsglvl,hsgt); 
	delay(gstab);
	rgpulse(pw,zero,rof1,rof1);
	zgradpulse(hsglvl,hsgt); 
	delay(0.05);
	}
	delay(d1);
status(B);
	rgpulse(pw,v1,rof1,0.0);
	obspower(pplvl);	
	if (tpwrf!=pplvlf) obspwrf(pplvlf);
	delay(0.25*kp_npoints/sw-rof1-gt1-gstab -POWER_DELAY);
	zgradpulse(gzlvl1,gt1);
	delay(gstab);
	rgpulse(pp,v2,rof1,rof1);
	obspower(pwr180_a); 
	if (tpwrf!=pplvlf) obspwrf(4095.0);
	if ( (0.25*kp_npoints/sw-rof1-gt1-gstab  -POWER_DELAY) > gstab )
	{
	delay(gstab);
	zgradpulse(gzlvl1,gt1);
	delay(0.25*kp_npoints/sw-rof1-gt1-gstab  -POWER_DELAY);
	}
	else
	{
	delay(0.25*kp_npoints/sw-rof1-gt1-gstab  -POWER_DELAY);
	zgradpulse(gzlvl1,gt1);
	delay(gstab);
	}
	delay(droppts1/sw);
	delay(tau_a-rof1-gt2-gstab-2.0*kp_pfgtc +rof2);
	zgradpulse(gzlvl2,gt2);
	delay(gstab);
	if (gzlvl7>0.0)
	{
	rgradient('z',gzlvl7);
	delay(kp_pfgtc);
	}
	shaped_pulse(shp_a,pw180_a,v3,rof1,rof1);
	if (gzlvl7>0.0)
	{
	rgradient('z',0.0);
	delay(kp_pfgtc);
	}
	if ((tau_a-rof1-gt2-gstab) > gstab)
	{
        delay(gstab);
	zgradpulse(gzlvl2,gt2);
	delay(tau_a-rof1-gt2-gstab);
	}
	else
	{
	delay(tau_a-rof1-gt2-gstab);
	zgradpulse(gzlvl2,gt2);
        delay(gstab);
	}
status(C);
	delay(1.0/(getval("fb")*1.3));
	obsblank();	
	delay(rof2);
	startacq(getval("alfa"));



starthardloop(v10);
	acquire(droppts1+kp_npoints+droppts2,1.0/sw);
	recoff();	
	obspower(pplvl);
	if (tpwrf!=pplvlf) obspwrf(pplvlf);
	obsunblank();

	delay(rof1-POWER_DELAY);

	 zgradpulse(1.0*ACQ_gzlvl1,ACQ_gt1);
	 delay(ACQ_gstab);


	rgpulse(pp,v4,rof1,rof1);
	delay(rof1);

	 zgradpulse(1.0*ACQ_gzlvl1,ACQ_gt1);
	 delay(ACQ_gstab);

	obspower(pwr180_a);
	delay((droppts1+droppts2)/sw);
	delay(tau_r-POWER_DELAY);

	 zgradpulse(1.0*ACQ_gzlvl2,ACQ_gt1);
	 delay(ACQ_gstab);

	if (gzlvl7>0.0)
	{
	rgradient('z',gzlvl7);
	delay(kp_pfgtc);
	}
	shaped_pulse(shp_a,pw180_a,v4,rof1,rof1);
	if (gzlvl7>0.0)
	{
	rgradient('z',0.0);
	delay(kp_pfgtc);
	}
	if (ACQ_gstab>tau_r)
	{

	 delay(tau_r-rof3);
	 zgradpulse(1.0*ACQ_gzlvl2,ACQ_gt1);
	 delay(ACQ_gstab);
	}
	else
	{

	 delay(ACQ_gstab);
	 zgradpulse(1.0*ACQ_gzlvl2,ACQ_gt1);
	 delay(tau_r-rof3);
	}
	rcvron();
endhardloop(); 
	acquire(droppts1+kp_npoints+droppts2,1.0/sw);
	recoff();
	endacq();	

}



